var getParameterFromURL = function getParameterFromURL(sParam) {
  var sPageURL = decodeURIComponent(window.location.search.substring(1)),
  sURLVariables = sPageURL.split('&'),
  sParameterName,
  i;

  for (i = 0; i < sURLVariables.length; i++) {
    sParameterName = sURLVariables[i].split('=');

    if (sParameterName[0] === sParam) {
      return sParameterName[1];
    }
  }
  return null;
};

var hasRoom = function hasRoom() {
  return (getParameterFromURL('room') !== null) && (getParameterFromURL('room') !== '');
}
var getRoom = function getRoom() {
  return getParameterFromURL('room');
}
var isVideoCall = function isVideoCall() {
  return (getParameterFromURL('video') !== null);
}

if (!hasRoom()) {
  location.href = "http://99.4.70.120/yalloo/";
}

var SIGNALING_SERVER = "http://99.4.70.120/";
var USE_AUDIO = true;
var IS_VIDEO_CALL = isVideoCall();
var CHANNEL_NAME = getRoom();
var MUTE_AUDIO_BY_DEFAULT = false;

var ICE_SERVERS = [
    {url:"stun:stun.l.google.com:19302"}
];


console.log('room: '+getParameterFromURL('room'));
console.log('IS_VIDEO_CALL: '+IS_VIDEO_CALL);
var socket = null;
var local_media_stream = null;
var users = {};
var user_media_elements = {};

function signalServer() {
    console.log("Connecting to signaling server");
    var options = {
      'force new connection': true,
      'reconnectrionAttempts': 'Infinity',
      'timeout': 10000,
      'transports': ['websocket'],
      // 'path': '/yalloo/session'
    };
    socket = io(SIGNALING_SERVER, options);

    socket.on('connect', function() {
        console.log("Connected to signaling server");
        setup_local_media(function() {
            join_chat_channel(CHANNEL_NAME);
        });
    });
    socket.on('disconnect', function() {
        console.log("Disconnected from signaling server");
        for (userId in user_media_elements) {
            user_media_elements[userId].remove();
        }
        for (userId in users) {
            users[userId].close();
        }

        users = {};
        user_media_elements = {};
    });
    function join_chat_channel(channel) {
        socket.emit('joined', {"channel": channel});
    }
    function leave_chat_channel(channel) {
        socket.emit('left', channel);
    }


    socket.on('addUser', function(config) {
        var userId = config.userId;
        var user_connection = new RTCuserConnection(
            {"iceServers": ICE_SERVERS},
            {"optional": [{"DtlsSrtpKeyAgreement": true}]}
        );
        users[userId] = user_connection;

        user_connection.onicecandidate = function(event) {
            if (event.candidate) {
                socket.emit('relayICECandidate', {
                    'userId': userId,
                    'iceCandidate': {
                        'sdpMLineIndex': event.candidate.sdpMLineIndex,
                        'candidate': event.candidate.candidate
                    }
                });
            }
        }
        user_connection.onaddstream = function(event) {
            console.log("onAddStream", event);
            var remote_media = IS_VIDEO_CALL ? $("<video>") : $("<audio>");
            remote_media.attr("autoplay", "autoplay");
            if (MUTE_AUDIO_BY_DEFAULT) {
                remote_media.attr("muted", "true");
            }
            // remote_media.attr("controls", "");
            remote_media.attr("value", "");
            user_media_elements[userId] = remote_media;
            $('.mini').append(remote_media);
            remote_media[0].srcObject = event.stream;
          }


        user_connection.addStream(local_media_stream);

        if (config.should_create_offer) {
            console.log("Creating RTC offer to ", userId);
            user_connection.createOffer(
                function (local_description) {
                    console.log("Local offer description is: ", local_description);
                    user_connection.setLocalDescription(local_description,
                        function() {
                            socket.emit('relaySessionDescription',
                                {'userId': userId, 'session_description': local_description});
                            console.log("Offer setLocalDescription succeeded");
                        },
                        function() { Alert("Offer setLocalDescription failed!"); }
                    );
                },
                function (error) {
                    console.log("Error sending offer: ", error);
                });
        }
    });


    socket.on('sessionDescription', function(config) {
        console.log('Remote description received: ', config);
        var userId = config.userId;
        var user = users[userId];
        var remote_description = config.session_description;
        console.log(config.session_description);

        var desc = new RTCSessionDescription(remote_description);
        var stuff = user.setRemoteDescription(desc,
            function() {
                console.log("setRemoteDescription succeeded");
                if (remote_description.type == "offer") {
                    console.log("Creating answer");
                    user.createAnswer(
                        function(local_description) {
                            console.log("Answer description is: ", local_description);
                            user.setLocalDescription(local_description,
                                function() {
                                    socket.emit('relaySessionDescription',
                                        {'userId': userId, 'session_description': local_description});
                                    console.log("Answer setLocalDescription succeeded");
                                },
                                function() { Alert("Answer setLocalDescription failed!"); }
                            );
                        },
                        function(error) {
                            console.log("Error creating answer: ", error);
                            console.log(user);
                        });
                }
            },
            function(error) {
                console.log("setRemoteDescription error: ", error);
            }
        );
        // console.log("Description Object: ", desc);

    });

    socket.on('iceCandidate', function(config) {
        var user = users[config.userId];
        var iceCandidate = config.iceCandidate;
        user.addIceCandidate(new RTCIceCandidate(iceCandidate));
    });

    socket.on('removeUser', function(config) {
        // console.log('Signaling server said to remove user:', config);
        var userId = config.userId;
        if (userId in user_media_elements) {
            user_media_elements[userId].remove();
        }
        if (userId in users) {
            users[userId].close();
        }

        delete users[userId];
        delete user_media_elements[config.userId];
    });
}// end init



function setup_local_media(callback, errorback) {
    if (local_media_stream != null) {
        if (callback) callback();
        return;
    }
    // console.log("Requesting access to local audio / video inputs");

    navigator.getUserMedia = ( navigator.getUserMedia ||
           navigator.webkitGetUserMedia ||
           navigator.mozGetUserMedia ||
           navigator.msGetUserMedia);
    var options = (IS_VIDEO_CALL === true)?{"audio":true, "video":true}:{"audio":true, "video":false};
    navigator.getUserMedia(options,
        function(stream) {
            // console.log("Access granted to audio/video");
            local_media_stream = stream;
            var local_media = IS_VIDEO_CALL ? $("<video>") : $("<audio>");
            local_media.attr("autoplay", "autoplay");
            local_media.attr("muted", "false");
            // local_media.attr("controls", "");
            $('.mini').append(local_media);
            local_media[0].srcObject = stream;

            if (callback) callback();
        },
        function() {
            // console.log("Access denied for audio/video");
            alert("You chose not to provide access to the camera/microphone, connection will not be established.");
            if (errorback) errorback();
        });
}
